__author__ = 'michey'
from flask import Flask
from config_adapter import ConfigAdapter
from container_pool import ContainerPool
import sys

def run():
    app = Flask(__name__)


    config_path = sys.argv[1]
    config = ConfigAdapter(config_path)
    containers = ContainerPool(config.get_socket_path, config.get_container,
        config.get_run_command, config.get_start_port)

    @app.route('/run')
    def run_node():
        containers.instantiate_node()
        return "ok;"

    @app.route('/list')
    def list_node():
        return containers.get_node_list()



from docker_adapter import DockerAdapter
from string import Template


class ContainerPool:
    """Class for managing pool of containers"""
    ports_pool = []

    def __init__(self, socket_path, image_path, run_command, start_port):
        self.adapter = DockerAdapter(socket_path)
        self.image_path = image_path
        self.run_command = Template(run_command)
        self.start_port = start_port

    def instantiate_node(self):
        current_port = len(self.ports_pool) + self.start_port
        self.ports_pool.append(current_port)
        command = self.run_command.substitute(exposed_port = current_port)
        self.adapter.run_container(self.image_path, command, current_port)

    def get_node_list(self):
        return self.adapter.get_nodes()




from docker import Client
from docker.utils import utils


class DockerAdapter:
    """Adapter for docker api"""
    def __init__(self, base_url):
        self.dockerClient = Client(base_url=base_url)

    def run_container(self, image_path, run_command, port_bind):
        self.dockerClient.create_container(
            image=image_path,
            command=run_command, detach=True,
            host_config=utils.create_host_config(
                port_bindings={port_bind: ('127.0.0.1',)}
            )
        )

    def get_nodes(self):
        return self.dockerClient.containers()



import ConfigParser
import logging

class ConfigAdapter:
    logging.basicConfig(level=logging.DEBUG)
    log = logging.getLogger('config_parser')
    config = ConfigParser.ConfigParser()
    sections = []
    data = {}
    """Adapter for config file"""
    def __init__(self, path):
        self.config.read(path)
        self.sections = self.config.sections()
        options = self.config.options(self.sections[0])
        for option in options:
            try:
                self.data[option] = self.config.get(self.sections[0], option)
                if self.data[option] == -1:
                    self.log.debug("skip: %s" % option)
            except:
                print("exception on %s!" % option)
                self.data[option] = None


    def get_socket_path(self):
        return self.data['socket_path']

    def get_container(self):
        return self.data['container']

    def get_run_command(self):
        return self.data['run_command']

    def get_start_port(self):
        return self.data['start_port']


from distutils.core import setup

setup(
    name='phjs-manager',
    version='0.0.15',
    author="michey",
    author_email='mixeyy@gmail.com',
    packages=["app"],
    include_package_data=True,
    url="http://pypi.python.org/pypi/MyApplication_v010/",
    description="phjs-manager",
    install_requires=['docker-py', 'flask'],
    entry_points={
    'console_scripts': [
        'phjs-manager = app:run',
    ],
},
)